LSQ-ML - Iterative least-squares maximum-likelihood ptychography solver
=======================================================================
version 20180116


Overview
-----------

This source code provides an implementation of the LSQ-ML method described in [1] in a simple Matlab-based framework. The reference implementation includes an artificial data generator and implementations of the ePIE [2], ePIE-OPRP [3] and DM [4] codes that were used in article [1]. 
The code is using Matlab-based GPU acceleration, however seemless fallback CPU option is available as well.


The source code is available at [www.psi.ch/sls/csaxs/software](http://www.psi.ch/sls/csaxs/software)




Requirements
------------

We recommend Matlab 2016a with CUDA 9 and newer.
Matlab parallel toolbox is required for GPU acceleration. 


Instalation
------------

Mex files compiled for Matlab 2017b and CUDA 9 are provided, in case of problems it can be recompiled by the following commands

GPU CUDA-based projectors::

    `mexcuda -output +core/  +core/get_projections_gpu_mex.cu`
    `mexcuda -output +core/  +core/set_projections_gpu_mex.cu`

CPU MPI-based projectors::

    `mex -outdir +core/ -O COPTIMFLAGS='-O2' LDOPTIMFLAGS='-O2' CFLAGS="\$CFLAGS -fopenmp" LDFLAGS="\$LDFLAGS -fopenmp"  +core/get_projections_cpu_mex.c`
    `mex -outdir +core/ -O COPTIMFLAGS='-O2' LDOPTIMFLAGS='-O2' CFLAGS="\$CFLAGS -fopenmp" LDFLAGS="\$LDFLAGS -fopenmp"  +core/set_projections_cpu_mex.c`
    `mex -outdir +core/ -O COPTIMFLAGS='-O2' LDOPTIMFLAGS='-O2' CFLAGS="\$CFLAGS -fopenmp" LDFLAGS="\$LDFLAGS -fopenmp"  +core/set_projections_cpu_mex_rc.c`


Structure
------------

Code is separated into: 

* artificial data creator 

* multiengine ptychography solver 
    1. *MLs/MLc* is **iterative least squares maximum likelihood method** with sparse / compact subset selection 
    2. *ePIE* is **extended ptychographic iterative engine**
    3. *ePIE_OPR* is **orthogonal probe relaxation method based on ePIE**
    4. *DM* is **difference maps**


probe is provided as an cell array of single precision complex arrays::

    `inputs.probe{incoherent_mode_number}[x,y,OPRP_probe_mode]`
    
where incoherent_mode_number refer to coherence modes described in [5], while OPRP_probe_mode are orthogonal modes described in [1,3]

Object is stored in cell array of single precision complex arrays::

    `inputs.object{scan_id}[x,y]`

Other important variables are provided in the @Data class

                 Np_p: 2x1 vector, size of single view 
                 Np_o: 2x1 vector, size of object 
                 Npos: number of scan positions
                probe: cell array of probe arrays with size Np_p
        probe_support: binary mask for probe support with size Np_p
    probe_positions_0: Npos x 2 array of initial probe positions estimates 
      probe_positions: [] or Npos x 2 array of refined probe positions 
          diffraction: Np_p(1) x Np_p(2) x Npos array of measured intensities
      reconstruct_ind: cell array of position indices used for reconstruction
               object: cell array of object arrays with size Np_o
          object_orig: original object used to generate data
           probe_orig: original probe used to generate data 
                modes: axiliary structure 
      probe_evolution: Npos x Nmodes array, evolution of the incoherent probe modes 


Run our example code 
-------------------------

Open our example template `ptychography_artificial.m` in Matlab and run it. 


Reconstruction parameters 
-------------------------

    param.use_gpu = true; % _use GPU (if availible)
    param.keep_on_gpu = true;  % keep data on GPU during processing, relevant only for DM engine 
    param.gpu_id = []; % used GPU id, [] means chosen by matlab

##### PIE / LSQML 
    % betas are step sizes, faster convergence vs higher stability
    param.beta_object = 1;
    param.beta_probe = 1;    
    param.beta_LSQ = true;   % use LSQ preditive step for ML method 
    param.delta_p = 0.1;     % LSQ damping constant , used only for MLs method

#####  DM
    param.pfft_relaxation = 0.05;  % relaxed modulus constraint for DM solver 
    param.probe_inertia = 0.1; % add inertia to the probe reconstruction to avoid oscilations 
#####  General
    param.Nprobes = 1; % mumber of probes 
    param.probe_reconstruct = 1;  % iteration when the probe reconstruction is started
    param.object_reconstruct = 1;% iteration when the object reconstruction is started
    param.Niter = 300 ;       % number of iterations for the chosen engine 
    param.grouping = 100;     % size of the processed blocks 
    param.method = 'MLs';     % choose reconstruction method: ePIE,ePIE_OPR, DM, MLs, MLc 
    param.verbose_level = 0;  % verbosity of the solver , 0 = quite, 4 = all 
    param.plot_results_every = 50;
    param.probe_support_radius = [];  % radius of circular probe suppport, [] = none, useful for DM code 
    param.probe_support_distance = 0; % distance of the probe support from the object plane, 0 = none, inf = farfield

#####  Advanced LSQML/OPRP options
    param.variable_probe = false;           % Use SVD to account for variable illumination during a single scan (Orthogonal probe relaxation)
    param.variable_probe_modes = 1;         % Number of SVD modes used for the orthogonal probe relaxation, ML method allows only one variable mode ! 
    param. variable_intensity = false;      % Account for variable illumination intensity during single scan
    param.apply_subpix_shift = false;       % apply FFT-based subpixel shift, important for good position refinement but it is slow
    param.probe_pos_search = inf;            % iteration number from which position correction is started 
    param.apply_multimodal_update = false; % apply higher Thibault's modes to the object update, may result in lower object quality for real datasets



References
----------

[1] M. Odstrcil, A. Menzel, M. Guizar-Sicairos, *Opt. Express* (2018)

[2] A. M. Maiden and J. M. Rodenburg, *Ultramicroscopy* **109**, 1256–1262 (2009)

[3] M. Odstrcil, P. Baksh, S. Boden, R. Card, J. Chad, J. Frey, and W. Brocklesby,  *Opt. Express* **24**, 8360–8369 (2016)

[4] P. Thibault, M. Dierolf et al., *Science* **321**, 7 (2009)

[5] P. Thibault and A. Menzel, *Nature* **494**, 68 (2013)





>  Academic License Agreement
>================================
>
> Introduction 
> •	This license agreement sets forth the terms and conditions under which the PAUL SCHERRER INSTITUT (PSI), CH-5232 Villigen-PSI, Switzerland (hereafter "LICENSOR") 
>   will grant you (hereafter "LICENSEE") a royalty-free, non-exclusive license for academic, non-commercial purposes only (hereafter "LICENSE") to use the cSAXS 
>   ptychography MATLAB package computer software program and associated documentation furnished hereunder (hereafter "PROGRAM").
>
> Terms and Conditions of the LICENSE
> 1.	LICENSOR grants to LICENSEE a royalty-free, non-exclusive license to use the PROGRAM for academic, non-commercial purposes, upon the terms and conditions 
>       hereinafter set out and until termination of this license as set forth below.
> 2.	LICENSEE acknowledges that the PROGRAM is a research tool still in the development stage. The PROGRAM is provided without any related services, improvements 
>       or warranties from LICENSOR and that the LICENSE is entered into in order to enable others to utilize the PROGRAM in their academic activities. It is the 
>       LICENSEE’s responsibility to ensure its proper use and the correctness of the results.”
> 3.	THE PROGRAM IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR 
>       A PARTICULAR PURPOSE AND NONINFRINGEMENT OF ANY PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS. IN NO EVENT SHALL THE LICENSOR, THE AUTHORS OR THE COPYRIGHT 
>       HOLDERS BE LIABLE FOR ANY CLAIM, DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES OR OTHER LIABILITY ARISING FROM, OUT OF OR IN CONNECTION WITH THE PROGRAM OR THE USE 
>       OF THE PROGRAM OR OTHER DEALINGS IN THE PROGRAM.
> 4.	LICENSEE agrees that it will use the PROGRAM and any modifications, improvements, or derivatives of PROGRAM that LICENSEE may create (collectively, 
>       "IMPROVEMENTS") solely for academic, non-commercial purposes and that any copy of PROGRAM or derivatives thereof shall be distributed only under the same 
>       license as PROGRAM. The terms "academic, non-commercial", as used in this Agreement, mean academic or other scholarly research which (a) is not undertaken for 
>       profit, or (b) is not intended to produce works, services, or data for commercial use, or (c) is neither conducted, nor funded, by a person or an entity engaged 
>       in the commercial use, application or exploitation of works similar to the PROGRAM.
> 5.	LICENSEE agrees that it shall make the following acknowledgement in any publication resulting from the use of the PROGRAM or any translation of the code into 
>       another computing language:
>       "Data processing was carried out using the cSAXS ptychography MATLAB package developed by the Science IT and the coherent X-ray scattering (CXS) groups, Paul 
>       Scherrer Institut, Switzerland."
>
>    Additionally, any publication using the package, or any translation of the code into another computing language should cite for difference map:
> P. Thibault, M. Dierolf, A. Menzel, O. Bunk, C. David, F. Pfeiffer, High-resolution scanning X-ray diffraction microscopy, Science 321, 379–382 (2008). 
>   (doi: 10.1126/science.1158573),
> for mixed coherent modes:
> P. Thibault and A. Menzel, Reconstructing state mixtures from diffraction measurements, Nature 494, 68–71 (2013). (doi: 10.1038/nature11806),
> for LSQ-ML method 
>  M. Odstrcil, A. Menzel, M. Guizar-Sicairos, Iterative least-squares solver for generalized maximum-likelihood ptychography, Opt. Express, in press (2018). (doi: ).
> for OPRP method 
>  M. Odstrcil, P. Baksh, S. A. Boden, R. Card, J. E. Chad, J. G. Frey, W. S. Brocklesby,  Ptychographic coherent diffractive imaging with orthogonal probe relaxation, 
>    Opt. Express 24, 8360 (2016). (doi: 10.1364/OE.24.008360).
> 6.	Except for the above-mentioned acknowledgment, LICENSEE shall not use the PROGRAM title or the names or logos of LICENSOR, nor any adaptation thereof, nor the 
>       names of any of its employees or laboratories, in any advertising, promotional or sales material without prior written consent obtained from LICENSOR in each case.
> 7.	Ownership of all rights, including copyright in the PROGRAM and in any material associated therewith, shall at all times remain with LICENSOR, and LICENSEE 
>       agrees to preserve same. LICENSEE agrees not to use any portion of the PROGRAM or of any IMPROVEMENTS in any machine-readable form outside the PROGRAM, nor to 
>       make any copies except for its internal use, without prior written consent of LICENSOR. LICENSEE agrees to place the following copyright notice on any such copies: 
>       © All rights reserved. PAUL SCHERRER INSTITUT, Switzerland, Laboratory for Macromolecules and Bioimaging, 2017. 
> 8.	The LICENSE shall not be construed to confer any rights upon LICENSEE by implication or otherwise except as specifically set forth herein.
> 9.	DISCLAIMER: LICENSEE shall be aware that Phase Focus Limited of Sheffield, UK has an international portfolio of patents and pending applications which relate 
>       to ptychography and that the PROGRAM may be capable of being used in circumstances which may fall within the claims of one or more of the Phase Focus patents, 
>       in particular of patent with international application number PCT/GB2005/001464. The LICENSOR explicitly declares not to indemnify the users of the software 
>       in case Phase Focus or any other third party will open a legal action against the LICENSEE due to the use of the program.
> 10.	This Agreement shall be governed by the material laws of Switzerland and any dispute arising out of this Agreement or use of the PROGRAM shall be brought before 
>       the courts of Zürich, Switzerland. 
>
>
>
